/*
 * Copyright (c) 2001-2003 International Computer Science Institute
 * See LICENSE file for licensing, conditions, and warranties on use.
 *
 * DO NOT EDIT THIS FILE - IT IS PROGRAMMATICALLY GENERATED
 *
 * Generated by '../scripts/clnt-gen'.
 *
 * $XORP: xorp/xrl/interfaces/finder_xif.hh,v 1.9 2003/03/10 23:01:28 hodson Exp $
 */

#ifndef __XRL_INTERFACES_FINDER_XIF_HH__
#define __XRL_INTERFACES_FINDER_XIF_HH__

#undef XORP_LIBRARY_NAME
#define XORP_LIBRARY_NAME "XifFinder"

#include "libxorp/xlog.h"
#include "libxorp/callback.hh"

#include "libxipc/xrl.hh"
#include "libxipc/xrl_error.hh"
#include "libxipc/xrl_sender.hh"


class XrlFinderV0p1Client {
public:
    XrlFinderV0p1Client(XrlSender* s) : _sender(s) {}
    virtual ~XrlFinderV0p1Client() {}

    typedef XorpCallback2<void, const XrlError&, const string*>::RefPtr CB0;
    /**
     *  Send Xrl intended to:
     *  
     *  Register client with Finder. Once client has been registered Xrl
     *  registrations may proceed. Once Xrl registrations have have been
     *  completed the client needs to be enabled for the Finder to respond to
     *  queries about the client. Fails if target_name is already registered.
     *  The target_name must support the finder_client interface in order to be
     *  able to process messages from the finder.
     *  
     *  @param tgt_name Xrl Target name
     */
    bool send_register_finder_client(
	const char*	target_name, 
	const string&	target_name, 
	const string&	class_name, 
	const string&	in_cookie, 
	const CB0&	cb
    );

    typedef XorpCallback1<void, const XrlError&>::RefPtr CB1;

    bool send_unregister_finder_client(
	const char*	target_name, 
	const string&	target_name, 
	const CB1&	cb
    );

    typedef XorpCallback1<void, const XrlError&>::RefPtr CB2;
    /**
     *  Send Xrl intended to:
     *  
     *  Enable resolution of xrls associated with target. Disabling Xrls at
     *  present does not cause clients to flush state associated with target so
     *  cached entries in clients are unaffected by this Xrl. Caller must be
     *  client that registered Xrl.
     *  
     *  @param tgt_name Xrl Target name
     */
    bool send_set_finder_client_enabled(
	const char*	target_name, 
	const string&	target_name, 
	const bool&	enabled, 
	const CB2&	cb
    );

    typedef XorpCallback2<void, const XrlError&, const bool*>::RefPtr CB3;
    /**
     *  Send Xrl intended to:
     *  
     *  Get enabled state information associated with finder client.
     *  
     *  @param tgt_name Xrl Target name
     */
    bool send_finder_client_enabled(
	const char*	target_name, 
	const string&	target_name, 
	const CB3&	cb
    );

    typedef XorpCallback2<void, const XrlError&, const string*>::RefPtr CB4;
    /**
     *  Send Xrl intended to:
     *  
     *  Add resolved Xrl into system, fails if xrl is already registered.
     *  
     *  @param tgt_name Xrl Target name
     */
    bool send_add_xrl(
	const char*	target_name, 
	const string&	xrl, 
	const string&	protocol_name, 
	const string&	protocol_args, 
	const CB4&	cb
    );

    typedef XorpCallback1<void, const XrlError&>::RefPtr CB5;
    /**
     *  Send Xrl intended to:
     *  
     *  Remove xrl
     *  
     *  @param tgt_name Xrl Target name
     */
    bool send_remove_xrl(
	const char*	target_name, 
	const string&	xrl, 
	const CB5&	cb
    );

    typedef XorpCallback2<void, const XrlError&, const XrlAtomList*>::RefPtr CB6;
    /**
     *  Send Xrl intended to:
     *  
     *  Resolve Xrl
     *  
     *  @param tgt_name Xrl Target name
     */
    bool send_resolve_xrl(
	const char*	target_name, 
	const string&	xrl, 
	const CB6&	cb
    );

    typedef XorpCallback2<void, const XrlError&, const XrlAtomList*>::RefPtr CB7;
    /**
     *  Send Xrl intended to:
     *  
     *  Get list of registered Xrl targets
     *  
     *  @param tgt_name Xrl Target name
     */
    bool send_get_xrl_targets(
	const char*	target_name, 
	const CB7&	cb
    );

    typedef XorpCallback2<void, const XrlError&, const XrlAtomList*>::RefPtr CB8;
    /**
     *  Send Xrl intended to:
     *  
     *  Get list of Xrls registered by target
     *  
     *  @param tgt_name Xrl Target name
     */
    bool send_get_xrls_registered_by(
	const char*	target_name, 
	const string&	target_name, 
	const CB8&	cb
    );

    typedef XorpCallback2<void, const XrlError&, const XrlAtomList*>::RefPtr CB9;
    /**
     *  Send Xrl intended to:
     *  
     *  Get list of IPv4 hosts that clients should accept IPC requests from.
     *  
     *  @param tgt_name Xrl Target name
     */
    bool send_get_ipv4_permitted_hosts(
	const char*	target_name, 
	const CB9&	cb
    );

    typedef XorpCallback2<void, const XrlError&, const XrlAtomList*>::RefPtr CB10;
    /**
     *  Send Xrl intended to:
     *  
     *  Get list of IPv4 nets that clients should accept IPC requests from.
     *  
     *  @param tgt_name Xrl Target name
     */
    bool send_get_ipv4_permitted_nets(
	const char*	target_name, 
	const CB10&	cb
    );

    typedef XorpCallback2<void, const XrlError&, const XrlAtomList*>::RefPtr CB11;
    /**
     *  Send Xrl intended to:
     *  
     *  Get list of IPv6 hosts that clients should accept IPC requests from.
     *  
     *  @param tgt_name Xrl Target name
     */
    bool send_get_ipv6_permitted_hosts(
	const char*	target_name, 
	const CB11&	cb
    );

    typedef XorpCallback2<void, const XrlError&, const XrlAtomList*>::RefPtr CB12;
    /**
     *  Send Xrl intended to:
     *  
     *  Get list of IPv6 nets that clients should accept IPC requests from.
     *  
     *  @param tgt_name Xrl Target name
     */
    bool send_get_ipv6_permitted_nets(
	const char*	target_name, 
	const CB12&	cb
    );

protected:
    XrlSender* _sender;

private:
    void unmarshall0(
	const XrlError&	e, 
	XrlArgs*	a, 
	CB0		cb
    );

    void unmarshall1(
	const XrlError&	e, 
	XrlArgs*	a, 
	CB1		cb
    );

    void unmarshall2(
	const XrlError&	e, 
	XrlArgs*	a, 
	CB2		cb
    );

    void unmarshall3(
	const XrlError&	e, 
	XrlArgs*	a, 
	CB3		cb
    );

    void unmarshall4(
	const XrlError&	e, 
	XrlArgs*	a, 
	CB4		cb
    );

    void unmarshall5(
	const XrlError&	e, 
	XrlArgs*	a, 
	CB5		cb
    );

    void unmarshall6(
	const XrlError&	e, 
	XrlArgs*	a, 
	CB6		cb
    );

    void unmarshall7(
	const XrlError&	e, 
	XrlArgs*	a, 
	CB7		cb
    );

    void unmarshall8(
	const XrlError&	e, 
	XrlArgs*	a, 
	CB8		cb
    );

    void unmarshall9(
	const XrlError&	e, 
	XrlArgs*	a, 
	CB9		cb
    );

    void unmarshall10(
	const XrlError&	e, 
	XrlArgs*	a, 
	CB10		cb
    );

    void unmarshall11(
	const XrlError&	e, 
	XrlArgs*	a, 
	CB11		cb
    );

    void unmarshall12(
	const XrlError&	e, 
	XrlArgs*	a, 
	CB12		cb
    );

};

#endif /* __XRL_INTERFACES_FINDER_XIF_HH__ */
